#pragma once

#include <SFML/Graphics.hpp>
#include <SFML/System/Clock.hpp>
#include "texture.hpp"
#include "animator.hpp"
#include "spatialmap.hpp"
#include "levelmanager.hpp"

using matrix::Matrix;
using RGBA = uint32_t;

struct Raycaster {
  int $pitch=0;
  sf::Clock $clock;
  sf::Shader $brightness;
  TexturePack &$textures;
  double $pos_x = 0;
  double $pos_y = 0;

  // initial direction vector
  double $dir_x = -1;
  double $dir_y = 0;

  // the 2d raycaster version of camera plane
  double $plane_x = 0;
  double $plane_y = 0.66;
  sf::Texture $view_texture;
  sf::Sprite $view_sprite;

  std::unique_ptr<RGBA[]> $pixels = nullptr;

  int $width;
  int $height;
  int $screen_pos_x = RAY_VIEW_X;
  int $screen_pos_y = RAY_VIEW_Y;
  GameLevel $level;
  Matrix $map;
  std::unordered_map<DinkyECS::Entity, SpriteTexture> $sprites;
  std::vector<double> $zbuffer; // width
  Animator $anim;

  Raycaster(TexturePack &textures, int width, int height);

  void cast_rays();
  void draw_ceiling_floor();
  void draw_pixel_buffer();
  void sprite_casting(sf::RenderTarget& target);
  void draw(sf::RenderTarget& target);

  void sort_sprites(std::vector<int>& order, std::vector<double>& dist, int amount);

  void position_camera(float player_x, float player_y);
  void set_position(int x, int y);

  inline size_t pixcoord(int x, int y) {
    return ((y) * $width) + (x);
  }

  void set_level(GameLevel level);
  void init_shaders();
};
